import os
import sys
import logging
from .kaomi_client import ServerError
from .globals import EXIT_STATUS_RESOLVER


def display_output_and_exit(client_output, verbose_flag=False):
    """
    Dato un dizionario rappresentante lo stato di uscita del server, costruisce lo stato di uscita del client ed esce dallo script
    :param client_output:       dizionario che contiene le informazioni sull'output
    :param verbose_flag:        flag che indica se l'output deve essere verbose, nei casi di errore viene sempre considerato (True)

    WARNING: la funzione termina lo script

    """

    # ottengo il significato dell'exit status del client, per poterlo stampare all'utente nel caso l'output sia verbose
    status = client_output.get('status', -1)
    status_meaning = EXIT_STATUS_RESOLVER.get(status, '{}: Unknown'.format(status))

    # ottengo il flag che indica se ci sono stati errori
    error_flag = client_output.get('error', True)           # se non esiste il campo, c'è stato un errore

    # ottengo il message d'uscita e i dati
    message = client_output.get('message', '')
    data = client_output.get('data', '')

    # stampo l'output e stampo nel logging

    # si è verificato un errore
    if error_flag:
        # stampo gli errori sia sui log che sul
        print('{} \nError: {} \nData: {}'.format(status_meaning, message, data))
        logging.error('{}. --- Error: {}. --- Data: {}'.format(status_meaning, message, data))

    # non si è verificato un errore
    elif not error_flag:
        # scrivo sui log che è stata eseguita questa operazione con successo
        logging.info('{}. --- Output: {}. --- Data: {}'.format(status_meaning, message, data))

        if verbose_flag:
            # nel caso lo script sia stato chiamato con verbose, mostro anche l'output sul terminale
            print('{} \nOutput: {} \nData: {}'.format(status_meaning, message, data))

    # esco dallo script con l'exit status specificato
    sys.exit(status)


def upload_directory(kaomi_client, local_dir, remote_dir, dir_perms=None, file_perms=None, verbose=False):
    """
    Funzione per il caricamento di una directory
    Nota: all'interno della funzione non mi devo preoccupare dell'overwrite perché so che remote_dir è vuota
    :param kaomi_client:        client da utilizzare per l'upload
    :param local_dir:           cartella da uploadare
    :param remote_dir:          posizione di upload per la cartella specificata
    :param dir_perms:           permessi da assegnare alle cartelle
    :param file_perms:          permessi da assegnare ai file
    :param verbose:             indica se l'output deve essere verbose
    :throw RuntimeError:        (anche dal kaomi-client) per problemi generici della funzione o delle operazione di uplaod
    :throw ConnectionError:     (re-raise dalle operazioni del kaomi-client) per problemi di comunicazione con il server
    """
    if verbose: print('Local folder "{}" is being uploading in "{}"'.format(local_dir, remote_dir))

    if not os.path.isdir(local_dir) or os.path.islink(local_dir):
        # la cartella passata non è una cartella o è un link ad una cartella
        raise RuntimeError('Wrong argument: "{}" is not a valid directory'.format(local_dir))

    for entry in os.listdir(local_dir):
        # scorro tutte le entry presenti all'interno della cartella

        # ottengo il path assoluto dell'entry
        entry_abs_path = os.path.join(local_dir, entry)

        # salto le entry che non mi interessano
        if os.path.islink(entry_abs_path) or os.path.ismount(entry_abs_path):
            # tipologie di entry che non ci interessano, le saltiamo
            continue

        if os.path.isdir(entry_abs_path):
            # LA ENTRY È UNA DIRECTORY

            # ottengo la nuova cartella di destinazione per la dir considerata
            new_remote_dir = os.path.join(remote_dir, entry)

            # come prima cosa devo creare la cartella, i vari catch ad eccezioni vengono fatti dal chiamante
            server_resp = kaomi_client.create_directory(dest_dir=new_remote_dir, perms=dir_perms)
            if server_resp.get('status') != 0:
                # controllo se l'azione ha avuto successo, altrimenti sollevo un'eccezione
                raise ServerError(server_resp)

            # arrivato qui sono sicuro che la creazione ha avuto successo
            # allora posso chiamare ricorsivamente la funzione di upload sulla cartella
            upload_directory(kaomi_client=kaomi_client, local_dir=entry_abs_path, remote_dir=new_remote_dir, dir_perms=dir_perms,
                             file_perms=file_perms, verbose=verbose)

        elif os.path.isfile(entry_abs_path):
            # LA ENTRY È UN FILE

            # ottengo il path del file di destinazione
            new_remote_file = os.path.join(remote_dir, entry)

            # procedo con l'upload del file, i vari catch ad eccezioni vengono fatti dal chiamante
            server_resp = kaomi_client.upload_file(local_file=entry_abs_path, dest_file=new_remote_file, perms=file_perms)
            if server_resp.get('status') != 0:
                # controllo se l'azione ha avuto successo, altrimenti sollevo un'eccezione
                raise ServerError(server_resp)

            # se è impostato il verbose, segnalo il successo dell'operazione
            if verbose: print('Local file "{}" uploaded in "{}"'.format(entry_abs_path, new_remote_file))

    # ho terminato di caricare la cartella, segnalo il successo se verbose è attivo
    if verbose: print('Local folder "{}" uploaded in "{}"'.format(local_dir, remote_dir))


def get_client_output(server_output):
    """
    Funzione che dato il dizionario ottenuto come risposta, ne ricava le informazioni da fornire in output al client
    :param server_output:       dizionario ottenuto dal server come risposta
    :return:                    dizionario con la risposta del server
    """
    requested_fields = ['status', 'substatus', 'message', 'data']
    if not all(k in server_output for k in requested_fields):
        # non ci sono le chiavi che ci aspetteremmo nel dizionario del server, quindi errore nel client
        return {'status': 1, 'error': True, 'message': 'Missing fields in server response: {}'.format(list(server_output.keys())), 'data': ''}

    # ottengo i campi dal dizionario del server
    status, substatus, message, data = server_output.get('status'), server_output.get('substatus'), server_output.get('message'), server_output.get('data')

    # analizzo lo stato di uscita del server e converto il valore di uscita in quello del client
    if status == 0:
        # il comando ha avuto successo
        return {'status': 0, 'error': False, 'message': '(server {}:{}) {}'.format(status, substatus, message), 'data': data}
    elif status == 1:
        # problema di connessione
        return {'status': 2, 'error': True, 'message': '(server error {}:{}) {}'.format(status, substatus, message), 'data': data}
    elif status == 2 and (substatus == 1 or substatus == 2):
        # problema di autorizzazioni
        return {'status': 3, 'error': True, 'message': '(server error {}:{}) {}'.format(status, substatus, message), 'data': data}
    else:
        # problema applicativo
        return {'status': 4, 'error': True, 'message': '(server error {}:{}) {}'.format(status, substatus, message), 'data': data}


from . import __version__

PERMS_MASK = 0o777

SERVER_PORT_DEFAULT = 47000

API_PATHS = {
    "upload_file": "api/v1/files/upload",
    "create_directory": "api/v1/files/mkdir",
    "delete_filesystem_node": "api/v1/files/delete_node",
    "execute_action": "api/v1/actions/execute"
}

EXIT_STATUS_RESOLVER = {
    0: "0 = Success",
    1: "1 = Client Error",
    2: "2 = Server Error: comunication",
    3: "3 = Server Error: permissions",
    4: "4 = Server Error: application"
}

# la versione dell'agent viene presa dall'init
AGENT_VERSION = __version__



import os
import sys
import logging
import argparse
from .kaomi_client import KaomiClient, ServerError
from .globals import PERMS_MASK, SERVER_PORT_DEFAULT, API_PATHS
from .functions import get_client_output, upload_directory, display_output_and_exit


def check_perms(value):
    """
    Funzione di controllo dei permessi passati via linea di comando
    Viene applicata conversione e maschera
    :param value:                   valore passato come argomento del parametro indicante i permessi
    :throw ArgumentTypeError:       nel caso il permesso non sia un numero
    :return:                        permessi validi
    """

    try:
        permissions = int(value, 8)
    except ValueError:
        # il numero passato non è convertibile in binario
        raise argparse.ArgumentTypeError('Permissions must be specified as an octal number (eg. 0o777, 0o0777, 0777, 777, etc.)')

    # maschero i permessi per eliminare configurazioni non ammissibili
    perms = permissions & PERMS_MASK

    # vedo se la maschera ha modificato i permessi
    if perms != permissions:
        print('Warning: specified permissions have been masked to {}'.format(oct(perms)))

    return perms


def get_arg_parser():
    """
    Funzione per l'inizializzazione dell'argparser da utilizzare
    :return:            istanza dell'arg_parser
    """
    parser = argparse.ArgumentParser(description="""
    Kaomi deployer client script.
    This script can be used to deploy folders, files or to execute actions on a server that runs kaomi deployer.

    """, prog="python -m kaomi_client")

    requested_action = parser.add_mutually_exclusive_group(required=True)

    # aggiunta degli argomenti da linea di comanda
    parser.add_argument('-k', '--key', type=str, dest='apikey', metavar="apikey", required=True, help="apikey to be used for authentication.")
    parser.add_argument('-s', '--server', type=str, dest='server_host', metavar="server_host", required=True, help="server host.")
    parser.add_argument('-p', '--port', type=int, dest='server_port', metavar="server_port", required=False, default=SERVER_PORT_DEFAULT, help="port of the server to be deployed (default: %(default)s)")

    requested_action.add_argument('-d', '--directory', nargs=2, dest='directory', metavar=("local_dir", "remote_dir"), help="upload directory 'local_dir' to 'remote_dir'.")
    requested_action.add_argument('-f', '--file', nargs=2,  dest='file', metavar=("local_file", "remote_file"), help="upload file 'local_file' to 'remote_file'.")
    requested_action.add_argument('-a', '--action', dest='action', metavar="action_name", help="action to be executed")
    requested_action.add_argument('-x', '--delete', dest='delete', metavar="path", help="delete specified file or directory")

    parser.add_argument('-dp', '--dir-perms', type=check_perms, dest='dir_perms', metavar="permissions", required=False, help="permissions to be applied to uploaded folders (default: root folder ones)")
    parser.add_argument('-fp', '--file-perms', type=check_perms, dest='file_perms', metavar="permissions", required=False, help="permissions to be applied to uploaded files (default: root folder ones)")
    parser.add_argument('-o', '--overwrite', dest='overwrite_flag', action='store_true', required=False, help="specifies if folder/file that already exists on server should be deleted or not (default: false)")
    parser.add_argument('-v', '--verbose', dest='verbose_flag', action='store_true', required=False, help="verbose output (default: false)")

    return parser


##########################
########## MAIN ##########
##########################

# logging configuration
logging.basicConfig(filename="client_errors.log", format="[%(asctime)s][%(levelname)s][%(lineno)d][%(funcName)s] %(message)s", level=logging.INFO)

# inizializzo il parser per gli argomenti via linea di comando
parser = get_arg_parser()
args = parser.parse_args(sys.argv[1:])

# provo ad aprire una connessione con il server
try:
    kaomi_client = KaomiClient(server_host=args.server_host, server_port=args.server_port, apikey=args.apikey, api_paths=API_PATHS)
except ConnectionError as e:
    display_output_and_exit({'status': 2, 'message': 'Error establishing new connection', 'data': 'ConnectionError: {}'.format(e)})

# il funzionamento del main risente delle modifiche
if args.action is not None:
    # è stata richiesta l'esecuzione di un'azione
    try:
        # noinspection PyUnboundLocalVariable
        server_resp = kaomi_client.execute_action(action_name=args.action)
    except ConnectionError as e:
        display_output_and_exit({'status': 2, 'message': 'ConnectionError executing action', 'data': 'ConnectionError: {}'.format(e)})
    except RuntimeError as e:
        display_output_and_exit({'status': 1, 'message': 'RuntimeError executing action', 'data': 'RuntimeError: {}'.format(e)})
    # non ci sono stati errori nella richiesta, analizzo cosa ho ottenuto
    # noinspection PyUnboundLocalVariable
    display_output_and_exit(client_output=get_client_output(server_output=server_resp), verbose_flag=args.verbose_flag)

elif args.file is not None:
    # è stato richiesto l'upload di un file

    # ottengo il path del file di origine e quello del file di destinazione
    local_file, remote_file = args.file

    # come prima cosa controllo che il file locale esista
    if not os.path.isfile(local_file):
        # il file che si vuole uploadare non esiste, allora ritorno errore
        display_output_and_exit({'status': 1, 'message': 'Cannot find selected file', 'data': 'File: {}'.format(local_file)})

    # se il file esiste procedo al deployment
    try:
        # noinspection PyUnboundLocalVariable
        server_resp = kaomi_client.upload_file(local_file=local_file, dest_file=remote_file, overwrite=args.overwrite_flag, perms=args.file_perms)
    except ConnectionError as e:
        display_output_and_exit({'status': 2, 'message': 'ConnectionError uploading file', 'data': 'ConnectionError: {}'.format(e)})
    except RuntimeError as e:
        display_output_and_exit({'status': 1, 'message': 'RuntimeError uploading file', 'data': 'RuntimeError: {}'.format(e)})

    # non ci sono stati errori nella richiesta, analizzo cosa ho ottenuto
    # noinspection PyUnboundLocalVariable
    display_output_and_exit(client_output=get_client_output(server_output=server_resp), verbose_flag=args.verbose_flag)

elif args.directory is not None:
    # è stato richiesto l'upload di una directory

    local_dir, remote_dir = args.directory

    if not os.path.isdir(local_dir):
        # la directory che si vuole uploadare non esiste, allora ritorno errore
        display_output_and_exit({'status': 1, 'message': 'Cannot find selected directory', 'data': 'Dir: {}'.format(local_dir)})

    # come prima cosa provo a creare la cartella sul server in modo da ottenere eventuali errori
    # ad esempio se overwrite = false e la cartella esiste già, lo si comunica all'utente
    try:
        # noinspection PyUnboundLocalVariable
        server_resp = kaomi_client.create_directory(dest_dir=remote_dir, overwrite=args.overwrite_flag, perms=args.dir_perms)
    except ConnectionError as e:
        display_output_and_exit({'status': 2, 'message': 'ConnectionError uploading folder', 'data': 'ConnectionError: {}'.format(e)})
    except RuntimeError as e:
        display_output_and_exit({'status': 1, 'message': 'RuntimeError uploading folder', 'data': 'RuntimeError: {}'.format(e)})

    # procedo solo se la creazione della cartella ha avuto successo
    # noinspection PyUnboundLocalVariable
    if server_resp.get('status') != 0:
        # la creazione della cartella non ha avuto successo, allora esco
        display_output_and_exit(client_output=get_client_output(server_output=server_resp), verbose_flag=args.verbose_flag)

    try:
        # se la cartella è stata creata corretamente allora provo a caricare il suo contenuto
        upload_directory(kaomi_client=kaomi_client, local_dir=local_dir, remote_dir=remote_dir, dir_perms=args.dir_perms,
                         file_perms=args.file_perms, verbose=args.verbose_flag)
    except ConnectionError as e:
        display_output_and_exit({'status': 2, 'message': 'ConnectionError uploading folder. Folder on server might be inconsistent!', 'data': 'ConnectionError: {}'.format(e)})
    except RuntimeError as e:
        display_output_and_exit({'status': 1, 'message': 'RuntimeError uploading folder. Folder on server might be inconsistent', 'data': 'RuntimeError: {}'.format(e)})
    except ServerError as exc:
        server_error = exc.args[0]
        display_output_and_exit(get_client_output(server_output=server_error))

    display_output_and_exit({'status': 0, 'message': 'Operation concluded successfully', 'data': ''}, args.verbose_flag)

elif args.delete is not None:
    # è stata richiesta l'eliminazione di un file o una cartella
    file_to_delete = args.delete

    try:
        # noinspection PyUnboundLocalVariable
        server_resp = kaomi_client.delete_filesystem_node(path=file_to_delete)
    except ConnectionError as e:
        display_output_and_exit({'status': 2, 'message': 'ConnectionError deleting node', 'data': 'ConnectionError: {}'.format(e)})
    except RuntimeError as e:
        display_output_and_exit({'status': 1, 'message': 'RuntimeError deleting node', 'data': 'RuntimeError: {}'.format(e)})

    # noinspection PyUnboundLocalVariable
    display_output_and_exit(client_output=get_client_output(server_output=server_resp), verbose_flag=args.verbose_flag)
else:
    # non è stato riconosciuto cosa è stato richiesto
    display_output_and_exit({'status': 1, 'message': 'Client called without an operation'})


import os
import json
import requests
from requests_toolbelt.multipart import encoder
from .globals import AGENT_VERSION


class ServerError(Exception):
    """
    classe di eccezioni utilizzata per distinguere tra gli errori del server e quelli del client
    Quando si solleva l'eccezione va passato il dizionario contente la risposta del server
    """


class KaomiClient:
    """
    Classe per la comunicazione con il server da deplyare
    """
    __host = ""             # host con il quale comunicare
    __port = ""             # porta sulla quale ascolta il kaomi server
    __apikey = ""           # apikey con cui autenticarsi al server
    __server = ""           # indirizzo completo del server

    __user_agent = ""       # user-agent utilizzato dal client

    __api_paths = {}        # dizionario contenente i path delle varie API

    def __init__(self, server_host, server_port, apikey, api_paths):
        """
        Inizializzazione della comunicazione con il server
        :param server_host:     host con il quale comunicare
        :param server_port:     porta sulla quale ascolta il kaomi server
        :param apikey:          apikey con cui autenticarsi al server
        :param api_paths:       dizionario contenente le risoluzioni dei path delle api
        :throw ConnectionError: se qualcosa va storto con la connessione con il server
        :throw TypeError:       nel caso sia sbagliato il type di un parametro
        """
        # provo a connettermi al server per verificare se è raggiungibile e up
        server = "{}:{}".format(server_host, server_port)

        self.__user_agent = 'kaomi-python-client/{}'.format(AGENT_VERSION)

        try:
            # effettuo una chiamata get sull'index del server
            resp = requests.get(server, headers={'User-Agent': self.__user_agent})
        except requests.exceptions.RequestException as e:
            raise ConnectionError('Cannot connect to server: {}. Please check if it is correct and up. Exception: {}'.format(server, e))

        # controllo che il server risponda
        if not resp.status_code == 200:
            raise ConnectionError('Cannot connect to server: {}. Please check if it is correct and up.'.format(server))
        resp = resp.json()          # ottengo il contenuto della risposta
        if not (resp.get('status') == 0 and resp.get('substatus') == 0):
            raise ConnectionError('Server responds but with wrong status or substatus codes'.format(server))

        if not isinstance(api_paths, dict):
            raise TypeError('Server paths must be specified as a dictionary')

        # il server ha risposto, allora inizializzo lo stato interno della classe
        self.__apikey = apikey
        self.__host = server_host
        self.__port = server_port
        self.__server = server
        self.__api_paths = api_paths

    def __get_api_url(self, requested_api):
        """
        Funzione che data un api richiesta (che deve essere presente in __api_paths)
        :param requested_api:       api del quale si vuole ottenere l'url
        :throw RuntimeError:        se l'api non è definita
        :return:                    url completo, utilizzabile nella richiesta post
        """
        if requested_api not in self.__api_paths:
            # se non esiste la enrty nei path definiti, non possiamo ottenere l'url corretto
            raise RuntimeError('API URL for "{}" not found'.format(requested_api))

        # ottengo l'url
        api_url = '{}/{}'.format(self.__server, self.__api_paths.get(requested_api))

        return api_url

    def upload_file(self, local_file, dest_file, overwrite=False, perms=None):
        """
        Funzione per l'upload di un file sul server
        :param local_file:      path del file locale da caricare sul server, si suppone che esista
        :param dest_file:       path di destinazione del file
        :param overwrite:       flag che indica se il file debba essere sovrascritto nel caso esista già
        :param perms:           permessi da assegnare al file
        :throw ConnectionError:     nel caso ci siano problemi con la richiesta verso il kaomi server
        :throw RuntimeError:        per tutti gli altri problemi
        :return:                dizionario con la risposta del server
        """
        try:
            # ottengo l'url da utilizzare
            api_url = self.__get_api_url("upload_file")         # might raise RuntimeError

            # apro una nuova sessione
            session = requests.Session()

            if not os.path.isfile(local_file):
                raise RuntimeError('Cannot find local file "{}"'.format(local_file))

            with open(local_file, 'rb') as fd:
                perms = oct(perms) if perms is not None else None

                # costruisco il payload della richiesta
                payload = {
                    "file": (os.path.basename(local_file), fd, "application/octet-stream"),
                    "apikey": self.__apikey,
                    "dest_path": dest_file,
                    "overwrite": 'true' if overwrite else 'false',
                    "perms": perms         # perché l'encoding funzioni i permessi devono essere passati come stringa
                }

                # utilizzo di un multipart encoder per poter caricare il file non tutto insieme
                # questo è dato dal fatto di usare il parametro "data" della richiesta post e non quello "files"
                form = encoder.MultipartEncoder(payload)
                headers = {"Prefer": "respond-async", "Content-Type": form.content_type, 'User-Agent': self.__user_agent}

                try:
                    resp = session.post(api_url, headers=headers, data=form)
                except (requests.exceptions.RequestException, ConnectionError) as e:
                    raise ConnectionError('[{}: "{}"->"{}"] Something went wrong with request. Exception: {}'.format("upload_file", local_file, dest_file, e))
                finally:
                    # sia che l'operazione sia riuscita o meno, chiudo la sessione
                    session.close()

            return resp.json()         # might raise JSONDecodeError
        except RuntimeError as e:
            # aggiungo l'informazione su quale operazione ha sollevato l'errore
            raise RuntimeError('[{}: "{}"->"{}"] {}'.format("upload_file", local_file, dest_file, e))
        except json.JSONDecodeError as e:
            # il server non ha risposto con un json
            raise ConnectionError('[{}: "{}"->"{}"] Server response is not a JSON {}\nData: {}'.format("upload_file", local_file, dest_file, e, resp.text))

    def create_directory(self, dest_dir, overwrite=False, perms=None):
        """
        Funzione per la creazione di una cartella sul server
        :param dest_dir:            cartella da creare sul server
        :param overwrite:           indica se la cartella deve essere sovrascritta nel caso esista
        :param perms:               permessi da assegnare alla cartella
        :throw ConnectionError:     nel caso ci siano problemi con la richiesta verso il kaomi server
        :throw RuntimeError:        per tutti gli altri problemi
        :return:                    dizionario con la risposta del server
        """
        try:
            # ottengo l'url da utilizzare
            api_url = self.__get_api_url("create_directory")         # might raise RuntimeError

            perms = oct(perms) if perms is not None else None

            # costruisco il payload
            payload = {
                "apikey": self.__apikey,
                "path": dest_dir,
                "overwrite": 'true' if overwrite else 'false',
                "perms": perms}

            try:
                # effettuo la richiesta
                resp = requests.post(api_url, data=json.dumps(payload), headers={'User-Agent': self.__user_agent})
            except (requests.exceptions.RequestException, ConnectionError) as e:
                raise ConnectionError('[{}: "{}"] Something went wrong with request. Exception: {}'.format("create_directory", dest_dir, e))

            # analizzo e ritorno il risultato
            return resp.json()  # might raise JSONDecodeError
        except RuntimeError as e:
            # aggiungo l'informazione su quale operazione ha sollevato l'errore
            raise RuntimeError('[{}: "{}"] {}'.format("create_directory", dest_dir, e))
        except json.JSONDecodeError as e:
            # il server non ha risposto con un json
            raise ConnectionError('[{}: "{}"] Server response is not a JSON {}\nData: {}'.format("create_directory", dest_dir, e, resp.text))

    def delete_filesystem_node(self, path):
        """
        Funzione per l'eliminazione di un file o di una directory sul server
        :param path:    path del nodo da eliminare
        :throw ConnectionError:     nel caso ci siano problemi con la richiesta verso il kaomi server
        :throw RuntimeError:        per tutti gli altri problemi
        :return:        dizionario con la risposta del server
        """
        try:
            # ottengo l'url da utilizzare
            api_url = self.__get_api_url("delete_filesystem_node")         # might raise RuntimeError

            # costruisco il payload
            payload = {"apikey": self.__apikey, "path": path}

            try:
                # effettuo la richiesta
                resp = requests.post(api_url, data=json.dumps(payload), headers={'User-Agent': self.__user_agent})
            except (requests.exceptions.RequestException, ConnectionError) as e:
                raise ConnectionError('[{}: "{}"] Something went wrong with request. Exception: {}'.format("delete_filesystem_node", path, e))

            # analizzo e ritorno il risultato
            return resp.json()         # might raise JSONDecodeError
        except RuntimeError as e:
            # aggiungo l'informazione su quale operazione ha sollevato l'errore
            raise RuntimeError('[{}: "{}"] {}'.format("delete_filesystem_node", path, e))
        except json.JSONDecodeError as e:
            # il server non ha risposto con un json
            raise ConnectionError('[{}: "{}"] Server response is not a JSON {}\nData: {}'.format("delete_filesystem_node", path, e, resp.text))

    def execute_action(self, action_name):
        """
        Funzione per l'esecuzione di un'azione sul server
        :param action_name:         nome dell'azione da eseguire, i nomi sono definiti per apikey
        :throw ConnectionError:     nel caso ci siano problemi con la richiesta verso il kaomi server
        :throw RuntimeError:        per tutti gli altri problemi
        :return:                    dizionario con la risposta del server
        """
        try:
            # ottengo l'url da utilizzare
            api_url = self.__get_api_url("execute_action")         # might raise RuntimeError

            # costruisco il payload
            payload = {"apikey": self.__apikey, "action": action_name}

            try:
                # effettuo la richiesta
                resp = requests.post(api_url, data=json.dumps(payload), headers={'User-Agent': self.__user_agent})
            except (requests.exceptions.RequestException, ConnectionError) as e:
                raise ConnectionError('[{}: "{}"] Something went wrong with request. Exception: {}'.format("execute_action", action_name, e))

            # analizzo e ritorno il risultato
            return resp.json()         # might raise JSONDecodeError
        except RuntimeError as e:
            # aggiungo l'informazione su quale operazione ha sollevato l'errore
            raise RuntimeError('[{}: "{}"] {}'.format("execute_action", action_name, e))
        except json.JSONDecodeError as e:
            # il server non ha risposto con un json
            raise ConnectionError('[{}: "{}"] Server response is not a JSON {}\nData: {}'.format("execute_action", action_name, e, resp.text))



import setuptools
import os

with open("README-package.md", "r") as fh:
    long_description = fh.read()

def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in read(rel_path).split('\n'):
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1].strip()
    else:
        raise RuntimeError("Unable to find version string.")


setuptools.setup(
    name="kaomi-client",
    version=get_version('kaomi_client/__init__.py'),
    author="Linkspirit Team",
    author_email="tecnici@linkspirit.it",
    description="This package contains the client endpoint for the Kaomi deployer. Kaomi client has to be run on the deployer system.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.linkspirit.it/linkspirit/kaomi-deployer.git",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'requests',
        'requests-toolbelt',
    ],
    python_requires='>=3',
)
